#ifndef STRING_TO_H
#define STRING_TO_H

#include <sstream>

/**
 * No-op I/O manipulator.  This is used to provide default behavior to
 * <c>string_to<>()</c> that does not modify the input stream
 * <c>istrm</c>.
 *
 * @param[in] istrm input stream
 * @return unmodified input stream
 */
std::ios_base&
noop_iomanip(std::ios_base& istrm)
{
    return istrm;
}

/**
 * Convert the string <c>s</c> to parameterized type <c>T</c> and
 * store the result in <c>value</c>.  The string is converted by
 * extracting the value from a <c>std::istream</c> initialized with
 * the string.  If <c>exact</c> is <c>true</c>, all of the characters
 * in the string must be used; otherwise, trailing unused characters
 * are allowed.  If the string cannot be converted, <c>false</c> is
 * returned and <c>value</c> is unchanged; otherwise, <c>true</c> is
 * returned. <p>
 *
 * If the I/O manipulator function <c>fmanip</c> is specified, it will
 * be applied to the input stream before the value is extracted.  By
 * default, the default I/O manipulator function does not alter the
 * input stream. <p>
 *
 * For example, this function can be used to perform an exact convert
 * from hexadecimal as follows: <p>
 *
 * <code>
 *   string_to<unsigned int>("0x123B", value, true, std::hex);
 * </code>
 *
 * @param[in] s input stream
 * @param[out] value value of converted string
 * @param[in] exact whether string must be an exact match
 * @param[in] fmanip I/O manipulator
 * @return whether the conversion succeeded
 */
template<typename T> bool
string_to(const std::string& s,
          T& value,
          bool exact = true,
          std::ios_base& (*fmanip)(std::ios_base&) = noop_iomanip)
{
    bool rv = false;
    T tmp = T();

    /* C++11 always writes to the target variable even if a failure
     * occurs.  So for portablility, we first extract to "tmp" and
     * only if that succeeds do we set "value". */
    std::istringstream istrm(s);
    istrm >> fmanip >> tmp;
    if (!istrm.fail()) {
        if (!exact || istrm.eof()) {
            /* Success. */
            value = tmp;
            rv = true;
        }
    }

    return rv;
}

#endif  /*  STRING_TO_H  */
